
/***************************************************************************
 *                                                                         *
 *   DgCollectCallBlock - Asterisk Application for DigiVoice Cards         *
 *                                                                         *
 *   Copyright (c) 2007-2007, Digivoice Tecnologia em Eletronica Ltda      *
 *                                                                         *
 *   Author: DigiVoice Tecnologia em Eletronica Ltda                       *
 *   desenvolvimento@digivoice.com.br                                      *
 *                                                                         *
 *   This software is licensed under GPL                                   *
 *   Check LICENSE file for details                                        *
 *                                                                         *
 ***************************************************************************/

#include "asterisk.h"
#include "config.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 4693 $")

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <voicerlib/voicerlib.h>

#include "asterisk/lock.h"
#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/translate.h"
#include "asterisk/image.h"
#include "asterisk/options.h"
#include "asterisk/config.h"

#include "dgvchannel.h"

#define AST_MODULE "ael"

#ifndef CC_AST_HAS_VERSION_1_4
static char *tdesc = "Collect Call Block for DGV Trunk Application";
#endif

static char *app = "DgCollectCallBlock";

static char *synopsis = "Block Collect Call on a DGV Trunk";

static char *descrip = 
"  DgCollectCallBlock(): Block collect Call on a DGV Trunk. \n\n";

#ifndef CC_AST_HAS_VERSION_1_4
STANDARD_LOCAL_USER;

LOCAL_USER_DECL;
#endif

static int ccb_exec(struct ast_channel *chan, void *data)
{

    struct dgv_pvt *p = (struct dgv_pvt *)chan->tech_pvt;

#ifdef CC_AST_HAS_VERSION_1_4
    struct ast_module_user *u;
    
    u = ast_module_user_add(chan);
#else
	struct localuser *u;
	
    LOCAL_USER_ADD(u);
#endif

#ifdef CC_AST_HAS_VERSION_1_4
	if (!strcasecmp(chan->tech->type, "DGV")) 
#else
	if (!strcasecmp(chan->type, "DGV")) 
#endif
        p->flashing = 1;
	else
		ast_log(LOG_WARNING, "%s is not a DGV channel\n", chan->name);

#ifdef CC_AST_HAS_VERSION_1_4
    ast_module_user_remove(u);
#else
    LOCAL_USER_REMOVE(u);
#endif
    
	return 0;
}

#ifdef CC_AST_HAS_VERSION_1_4	
static int unload_module(void)
#else
int unload_module(void)
#endif
{
	int res;

	res = ast_unregister_application(app);
	
#ifdef CC_AST_HAS_VERSION_1_4	
    ast_module_user_hangup_all();
#else
	STANDARD_HANGUP_LOCALUSERS;
#endif	

	return res;
}

#ifdef CC_AST_HAS_VERSION_1_4	
static int load_module(void)
#else
int load_module(void)
#endif
{
	return ast_register_application(app, ccb_exec, synopsis, descrip);
}

#ifdef CC_AST_HAS_VERSION_1_4
AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Collect Call Block for DGV Trunk Application");
#else
char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}

#endif
