
/***************************************************************************
 *                                                                         *
 *   dgvchannel.h - Channel Driver for Asterisk (c)                        *
 *                                                                         *
 *   Copyright (c) 2006, Digivoice Tecnologia em Eletronica Ltda           *
 *                                                                         *
 *   Module: Asterisk Channel for Digivoice Cards                          *
 *                                                                         *
 *   Author: DigiVoice Tecnologia em Eletronica Ltda                       *
 *   desenvolvimento@digivoice.com.br                                      *
 *                                                                         *
 *   This program is free software, distributed under the terms of         * 
 *   the GNU General Public License Version 2. See the LICENSE file        *
 *   at the top of the source tree.                                        *
 *                                                                         *
 ***************************************************************************/

#ifndef __DGVCHANNEL_H__
#define __DGVCHANNEL_H__



#define TAMSTR				255		
#define MAX_SAMPLE_SIZE 	320

#define DGV_SAMPLES         160 
#define DGV_MAX_BUF         DGV_SAMPLES*25 + AST_FRIENDLY_OFFSET

#define NUM_SPANS_PER_CARD              2
#define NUM_DCHANS  (MAX_CARDS*NUM_SPANS_PER_CARD)

#define    TX      1
#define    RX      0


/* dgv ports states */
enum {
	DG_AST_NONE,
	DG_AST_RINGING,
	DG_AST_INCOMING_CALL,
	DG_AST_WAITFORDIALTONE,
	DG_AST_DIALING,
	DG_AST_CALLING,
	DG_AST_CONVERSATION,
	DG_AST_R2_SIGNALLING,
    DG_AST_FLASHING,
    DG_AST_HANGINGUP
};


#ifdef USE_LIBPRI
//--------------------------------------
// structure that holds pri information
//--------------------------------------
/* struct that holds pri information */
struct dgv_dchan {
    short card;
    short span;
    short pritype;
    short switchtype;
    short reseting;
    short alarmed;
    short overlapdial;
    unsigned int pridialplan;
    unsigned int prilocaldialplan;
    struct pri *pri;
    int fds[2];						             /*!< FD's for d-channels - using socketpairs */
};

struct dgv_dchan dchans[MAX_CARDS*NUM_SPANS_PER_CARD];
int dchans_count = 0;
static int fdmax=-1;                             // maximum file descriptor number

/* ISDN global variables */
//int pritype = PRI_CPE;
//int switchtype = PRI_SWITCH_EUROISDN_E1;
#endif


static struct dgv_pvt  {

	ast_mutex_t lock;			                 /* private lock */

#ifdef DEBUG_Q
	FILE *rec;
	FILE *play;
#endif


	char dev[256];				                 /* Device name, eg vpb/1-1 */
	short	board;					             /* board number */
	short ch_board;				                 /* relative port number - within the card*/


    /* debug purposes */
    int force_drop_tx;
    int force_drop_rx;

	short 	last_dtmf;			                 /* stores last dtmf and send it when receive silence */

    int pipe[2];

    short sig;            /* signalling used */

    char buf[DGV_MAX_BUF];			/* Static buffer for reading frames */

	char accountcode[AST_MAX_ACCOUNT_CODE];		//Account code
    char language[MAX_LANGUAGE];		/* language being used */

	short	port;						/* absolute port number */
	short group;					/* group this port belongs to */

	short write_last_format;     /* hold last play format */
	short read_last_format;     /* hold last record format */
	
	int nativeformats;

	ast_mutex_t play_lock;			/* mutex for playing */
	ast_mutex_t record_lock;		;			/* mutex for recording */

    pthread_t tone_thread_id;			/* pthread_t is unsigned long int */
	short tone_thread_running;		//flag indicating thread is running or not
	short tone_thread_type;				//type of tone - busy or ringback

  	//call control sctructure
	short call_state;

    short gen_ringback;             /* indicates if use or not ringback tone */
    short gen_busy;                 /* indicates if use or not busy tone */

    short flashing;                              //used into DgvChannel and DgCollectCallBlock

	char szNumberToDial[TAMSTR];

    char callprogress[TAMSTR];

	char context[AST_MAX_CONTEXT];
	int amaflags;

    char musicclass[TAMSTR];

    ast_group_t pickupgroup;

 	char exten[AST_MAX_EXTENSION];
	char cid_num[AST_MAX_EXTENSION];
	char rdnis[AST_MAX_EXTENSION];
	char dnid[AST_MAX_EXTENSION];
	char cid_name[TAMSTR];
	char pri_cid_num[AST_MAX_EXTENSION];

	//individual port configuration
	char szPortID[AST_MAX_EXTENSION];
	short category;
	short max_digits_rx;
    short min_digits_rx;
    //short auto_r2dmfc;
	short interdigit_timeout;
	short tx_gain;
	short rx_gain;
	short ask_for_cid_position;
	short echo_can;
    short echo_training;
    short drop_tx_samples_factor;                //factor to drop tx samples (8 = 160ms, etc..)

    short gen_dtmf;                              /* 0 - don't generate, 1 - generate */

    short dtmf_to_asterisk;                      /* 0 - don't generate, 1 - generate */

    short relaxdtmf;

    short b_subscriber_hangup;                   /* b subscriber can hangs up*/

    short group_b_value;                         /* group b to be sent when using ndis variable */

	short audio_debug;

    short verbosity;                             /* specific port verbosity */

	int   faxhandled;                            /* Has a fax tone already been handled? */
	int   fxo_bridged_dial;                      /* don't generate dtmf and silence(mute) when setted */

	short pref_format;				             //stores audio format

	short reserved;            					 /* channel reserved */

    short dialtone_timeout;                      //time to wait linetone
    short wait_dialtone;                         //wait for dialtone?
    short fax_detection;                         //enable fax detection?    

	int digit_timeout_sched;
    int linetone_timeout_sched;
	int ring_timeout_sched;                      //ast_sched to ring on FXO card
	int fxss_digit_timeout_sched;
	int all_echotraining_sched;
    
    short silence_threshold;                     /* value of silence threshold during conversation */
	
	struct ast_channel *ast_ch;		             /* store asterisk channel pointer or NULL */
	struct dgv_pvt *next;					     /* Next channel in list */
	struct dgv_pvt *previous;			         /* previous channel in list */

	//buffers to use in SamplesFromCard
	char  BufferRec8[MAX_SAMPLE_SIZE/2 + AST_FRIENDLY_OFFSET];
	short BufferRec16[MAX_SAMPLE_SIZE + AST_FRIENDLY_OFFSET];

	struct timeval tlastdigit;		/* used for digit timeout */

	unsigned long		samples_rx_count;

#ifdef USE_LIBPRI
	q931_call *call;
    struct pri *pri;
    char internationalprefix[10];					/*!< country access code ('00' for european dialplans) */
    char nationalprefix[10];					/*!< area access code ('0' for european dialplans) */
    char localprefix[20];						/*!< area access code + area code ('0'+area code for european dialplans) */
    char privateprefix[20];						/*!< for private dialplans */
    char unknownprefix[20];						/*!< for unknown dialplans */
	int cid_ton;					/*!< Type Of Number (TON) */
	//int immediate;
	int pri_channel;
    int alreadyhungup;              //controls if isdn channel is already hangup
    int stripmsd;
#endif

    int digital;
	int use_callerid;	
    char customcascfg[TAMSTR];
    
//for fxs cards
	int ring_type;
    int flash_min_time;	    		
	int flash_max_time;
	int immediate;
    char mailbox[AST_MAX_EXTENSION];    
	char fxss_exten[AST_MAX_EXTENSION];          //dtmf digits are readed in EV_DTMF
	int fxss_timeout;					         //timeout to fxs overlapdial
    pthread_t stuttertone_thread_id;			 /* pthread_t is unsigned long int */
	short stuttertone_thread_running;		     //flag indicating thread is running or not
		
} *iflast = NULL, *iffirst = NULL;

#endif
